@@ -50,7 +50,7 @@ from colorama import Fore, Style
 """
 
 
-class SharedCounter(object):
+class _SharedCounter(object):
     """ A synchronized shared counter.
 
     The locking done by multiprocessing.Value ensures that only a single
@@ -78,6 +78,31 @@ class SharedCounter(object):
         """ Return the value of the counter """
         return self.count.value
 
+class SharedCounter(object):
+    """ A synchronized shared counter.
+    The locking done by multiprocessing.Value ensures that only a single
+    process or thread may read or write the in-memory ctypes object. However,
+    in order to do n += 1, Python performs a read followed by a write, so a
+    second process may read the old value before the new one is written by the
+    first process. The solution is to use a multiprocessing.Lock to guarantee
+    the atomicity of the modifications to Value.
+    This class comes almost entirely from Eli Bendersky's blog:
+    http://eli.thegreenplace.net/2012/01/04/shared-counter-with-pythons-multiprocessing/
+    """
+
+    def __init__(self, n = 0):
+        self.count = multiprocessing.Value('i', n)
+
+    def increment(self, n = 1):
+        """ Increment the counter by n (default = 1) """
+        with self.count.get_lock():
+            self.count.value += n
+
+    @property
+    def value(self):
+        """ Return the value of the counter """
+        return self.count.value
+
 
 ################################################################################
 #
@@ -86,7 +111,7 @@ class SharedCounter(object):
 #Code license is GPLv3 according to github.com/vterron/lemon/setup.py
 #
 ################################################################################
-class py23_mp_queue(MP_Queue):
+class _py23_mp_queue(MP_Queue):
     """ A portable implementation of multiprocessing.Queue.
 
     Because of multithreading / multiprocessing semantics, Queue.qsize() may
@@ -104,16 +129,15 @@ class py23_mp_queue(MP_Queue):
         # Requires different __init__() based on Python2 vs Python3.4...
         if sys.version_info >= (3, 4, 0):
             super(py23_mp_queue, self).__init__(
-                *args, ctx=multiprocessing.get_context(), **kwargs
+                *args, ctx=multiprocessing.get_context("spawn"), **kwargs
             )
         else:
             super(py23_mp_queue, self).__init__(*args, **kwargs)
         self.size = SharedCounter(0)
+        print("HERE {}".format(self.size.value))
 
     def put(self, *args, **kwargs):
-        local_size = self.size
         super(py23_mp_queue, self).put(*args, **kwargs)
-        local_size.increment(1)
         self.size.increment(1)
 
     def get(self, *args, **kwargs):
@@ -128,6 +152,51 @@ class py23_mp_queue(MP_Queue):
         """Reliable implementation of multiprocessing.Queue.empty() """
         return not self.qsize()
 
+class py23_mp_queue(MP_Queue):
+    """ A portable implementation of multiprocessing.Queue.
+    Because of multithreading / multiprocessing semantics, Queue.qsize() may
+    raise the NotImplementedError exception on Unix platforms like Mac OS X
+    where sem_getvalue() is not implemented. This subclass addresses this
+    problem by using a synchronized shared counter (initialized to zero) and
+    increasing / decreasing its value every time the put() and get() methods
+    are called, respectively. This not only prevents NotImplementedError from
+    being raised, but also allows us to implement a reliable version of both
+    qsize() and empty().
+    """
+    # NOTE This is the new implementation based on:
+    #    https://github.com/vterron/lemon/blob/master/util/queue.py
+
+    def __init__(self, *args, **kwargs):
+        if sys.version_info >= (3, 4, 0):
+            super(py23_mp_queue, self).__init__(
+                *args, ctx=multiprocessing.get_context("spawn"), **kwargs
+            )
+        else:
+            super(py23_mp_queue, self).__init__(*args, **kwargs)
+        self._size = SharedCounter(0)
+
+    def put(self, *args, **kwargs):
+        super(py23_mp_queue, self).put(*args, **kwargs)
+        self._size.increment(1)
+
+    def get(self, *args, **kwargs):
+        item = super(py23_mp_queue, self).get(*args, **kwargs)
+        self._size.increment(-1)
+        return item
+
+    def qsize(self):
+        """ Reliable implementation of multiprocessing.Queue.qsize() """
+        return self._size.value
+
+    def empty(self):
+        """ Reliable implementation of multiprocessing.Queue.empty() """
+        return not self.qsize()
+
+    def clear(self):
+        """ Remove all elements from the Queue. """
+        while not self.empty():
+            self.get()
+
 
 class Worker(object):
     """multiprocessing worker"""
